/******************************************************************************
 *
 * Copyright:
 *    (C) 2005 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates how to install an irq handler,
 *    in this case, a FIQ handler.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include <framework.h>
#include "startup/config.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define CRYSTAL_FREQUENCY FOSC
#define PLL_FACTOR        PLL_MUL
#define VPBDIV_FACTOR     PBSD


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);


/*****************************************************************************
 * External function prototypes
 ****************************************************************************/
static void _fiqHandler(void) __attribute__ ((interrupt));


/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    installed C handle for SWI
 *
 ****************************************************************************/
static void
_fiqHandler(void)
{
	printf("\nFIQ interrupt!");
	
  //end interrupt
  EXTINT = 0x00000002;       //reset IRQ flag
  VICVectAddr = 0x00;        //dummy write to VIC to signal end of interrupt 
}


/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((CRYSTAL_FREQUENCY * PLL_FACTOR)/ (1000 * VPBDIV_FACTOR));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}


/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
  //initialize printf()-functionality
  eaInit();

  //print welcome message
  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2005 Embedded Artists AB");
  printf("\n*");
  printf("\n* Welcome to this program that demonstrates how to");
  printf("\n* install an irq handler, FIQ handler in this case.");
  printf("\n* The FIQ handler is triggered by pulling P0.14 low.");
  printf("\n* The pin is falling edge sensitive.");
  printf("\n*");
  printf("\n*********************************************************\n");

  //install own FIQ handler (other than default handler)
  pISR_FIQ = (unsigned int)_fiqHandler;

  //initialize P0.14 to EINT1 (active falling edge)
  EXTMODE  = 0x00000002;   //EINT1 is edge sensitive
  EXTPOLAR = 0x00000000;   //EINT1 is falling edge sensitive
  PINSEL0 &= ~0x30000000;
  PINSEL0 |=  0x20000000; 
  EXTINT   = 0x00000002;   //reset EINT1 IRQ flag

  //initialize VIC for EINT1 interrupts (as FIQ)
  VICIntSelect |= 0x00008000;      //EINT1 interrupt is assigned to FIQ (not IRQ)
  VICIntEnable  = 0x00008000;      //enable eint1 interrupt

  while(1)
  {
    printf(".");
  	
  	//wait 1000 ms
    delayMs(1000);
  }

  return 0;
}
